@@ -33,6 +33,8 @@ namespace charon {
   protected:
 
     Teuchos::RCP<charon::Names>  m_names;
+    //options
+    std::string UseFD="False";
 
   };
 
